/*
 *
 *    OPEN-XCHANGE legal information
 *
 *    All intellectual property rights in the Software are protected by
 *    international copyright laws.
 *
 *
 *    In some countries OX, OX Open-Xchange, open xchange and OXtender
 *    as well as the corresponding Logos OX Open-Xchange and OX are registered
 *    trademarks.
 *    The use of the Logos is not covered by the GNU General Public License.
 *    Instead, you are allowed to use these Logos according to the terms and
 *    conditions of the Creative Commons License, Version 2.5, Attribution,
 *    Non-commercial, ShareAlike, and the interpretation of the term
 *    Non-commercial applicable to the aforementioned license is published
 *    on the web site http://www.open-xchange.com/EN/legal/index.html.
 *
 *    Please make sure that third-party modules and libraries are used
 *    according to their respective licenses.
 *
 *    Any modifications to this package must retain all copyright notices
 *    of the original copyright holder(s) for the original code used.
 *
 *    After any such modifications, the original and derivative code shall remain
 *    under the copyright of the copyright holder(s) and/or original author(s)per
 *    the Attribution and Assignment Agreement that can be located at
 *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
 *    given Attribution for the derivative code and a license granting use.
 *
 *     Copyright (C) 2016 OX Software GmbH
 *     Mail: info@open-xchange.com
 *
 *
 *     This program is free software; you can redistribute it and/or modify it
 *     under the terms of the GNU General Public License, Version 2 as published
 *     by the Free Software Foundation.
 *
 *     This program is distributed in the hope that it will be useful, but
 *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *     for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc., 59
 *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

package com.openexchange.office.filter.core.component;

import com.openexchange.office.filter.core.DLNode;
import com.openexchange.office.filter.core.SplitMode;

public abstract class ComponentContext<O,C> implements IComponentContext<ComponentContext<O, C>, C> {

    final protected O operationDocument;
    private ComponentContext<O,C> parentContext;
    private DLNode<Object> node;

    public ComponentContext(O operationDocument, DLNode<Object> n) {
        this.operationDocument = operationDocument;
        parentContext = null;
        node = n;
    }

    public ComponentContext(ComponentContext<O,C> p, DLNode<Object> n) {
        operationDocument = p.operationDocument;
        parentContext = p;
        node = n;
    }

    public Object getObject() {
        return getNode().getData();
    }

    public O getOperationDocument() {
        return operationDocument;
    }

    @Override
    public ComponentContext<O,C> getParentContext() {
        return parentContext;
    }

    public void setParentContext(ComponentContext<O,C> p) {
        parentContext = p;
    }

    public DLNode<Object> getNode() {
        return node;
    }

    public void setNode(DLNode<Object> n) {
        node = n;
    }

    @Override
    public void splitStart(int n, SplitMode splitMode) {
        //
    }

    @Override
    public void splitEnd(int n, SplitMode splitMode) {
        //
    }

    @Override
    public abstract C getNextChildComponent(ComponentContext<O,C> previousChildContext, C previousChildComponent);

    public ComponentContext<O,C> getRootContext() {
        ComponentContext<O,C> rootContext;
        ComponentContext<O,C> parent = this;
        do {
            rootContext = parent;
            parent = parent.getParentContext();
        }
        while(parent!=null);

        return rootContext;
    }
}
