/*
*
*    OPEN-XCHANGE legal information
*
*    All intellectual property rights in the Software are protected by
*    international copyright laws.
*
*
*    In some countries OX, OX Open-Xchange, open xchange and OXtender
*    as well as the corresponding Logos OX Open-Xchange and OX are registered
*    trademarks.
*    The use of the Logos is not covered by the GNU General Public License.
*    Instead, you are allowed to use these Logos according to the terms and
*    conditions of the Creative Commons License, Version 2.5, Attribution,
*    Non-commercial, ShareAlike, and the interpretation of the term
*    Non-commercial applicable to the aforementioned license is published
*    on the web site http://www.open-xchange.com/EN/legal/index.html.
*
*    Please make sure that third-party modules and libraries are used
*    according to their respective licenses.
*
*    Any modifications to this package must retain all copyright notices
*    of the original copyright holder(s) for the original code used.
*
*    After any such modifications, the original and derivative code shall remain
*    under the copyright of the copyright holder(s) and/or original author(s)per
*    the Attribution and Assignment Agreement that can be located at
*    http://www.open-xchange.com/EN/developer/. The contributing author shall be
*    given Attribution for the derivative code and a license granting use.
*
*     Copyright (C) 2016 OX Software GmbH
*     Mail: info@open-xchange.com
*
*
*     This program is free software; you can redistribute it and/or modify it
*     under the terms of the GNU General Public License, Version 2 as published
*     by the Free Software Foundation.
*
*     This program is distributed in the hope that it will be useful, but
*     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
*     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
*     for more details.
*
*     You should have received a copy of the GNU General Public License along
*     with this program; if not, write to the Free Software Foundation, Inc., 59
*     Temple Place, Suite 330, Boston, MA 02111-1307 USA
*
*/
package com.openexchange.office.realtime.impl.calc;

import com.openexchange.office.realtime.impl.Connection;
import com.openexchange.office.realtime.impl.ConnectionComponent;
import com.openexchange.office.tools.rt.IDUtils;
import com.openexchange.realtime.exception.RealtimeException;
import com.openexchange.realtime.exception.RealtimeExceptionCodes;
import com.openexchange.realtime.packet.ID;
import com.openexchange.server.ServiceLookup;


/**
 * Class to control the life-time of spreadsheet connections.
 *
 * {@link CalcComponent}
 *
 * @author <a href="mailto:carsten.driesner@open-xchange.com">Carsten Driesner</a>
 */
public class CalcComponent extends ConnectionComponent {

    private static final String componentID = "spreadsheet";
    private final AsyncLoadRequestQueue m_requestQueue;

    public CalcComponent(ServiceLookup servicesDEPRECATED) {
        super(null, componentID);
        m_requestQueue = new AsyncLoadRequestQueue(null);
    }

    @Override
    public Connection createConnectionInstance(ID id) throws RealtimeException {
        if (null == limitLoadFactor) {
            throw RealtimeExceptionCodes.COMPONENT_HANDLE_CREATION_DENIED.create(id);
        }

        if (!limitLoadFactor.canCreateNewInstance()) {
            throw RealtimeExceptionCodes.COMPONENT_HANDLE_CREATION_DENIED.create(id);
        }

        return new CalcConnection(getServiceLookup(), id, m_requestQueue, componentID);
    }

    @Override
    protected void onConnectionCreated(ID id) {
        // start request queue on demand
        if ((getConnectionCount() > 0) && (!m_requestQueue.isStarted())) {
            m_requestQueue.startQueue();
        }
    }

    @Override
    protected void onConnectionDestroyed(ID id) {
        // purge possible requests from the queue
        final String resource = IDUtils.getResourceId(id);
        if (null != resource) {
            m_requestQueue.purgeDocumentRequests(Request.buildDocumentId(resource));
        }
    }

    @Override
    protected void onShutdown() {
        m_requestQueue.destroyQueue();
    }
}
